/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        main.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the CompanyE for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS ISECONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18f2450.h>
#include <delays.h>
#include "typedefs.h"                        // Required
#include "usb.h"                         // Required

#include "usb_compile_time_validation.h" // Optional

/** V A R I A B L E S ********************************************************/
#pragma romdata CONFIG
#define _CONFIG1L 0b00100100
#define _CONFIG1H 0b00001111
#define _CONFIG2L 0b00111110
#define _CONFIG2H 0x00
#define _CONFIG3H 0b00000000
#define _CONFIG4L 0b10000001
#define _CONFIG5L 0xFF
#define _CONFIG5H 0xFF
#define _CONFIG6L 0xFF
#define _CONFIG6H 0xFF
#define _CONFIG7L 0xFF
#define _CONFIG7H 0xFF
const rom unsigned char _configuration[14]={_CONFIG1L,_CONFIG1H,_CONFIG2L,_CONFIG2H,0xFF,_CONFIG3H,_CONFIG4L,0xFF,_CONFIG5L,_CONFIG5H,_CONFIG6L,_CONFIG6H,_CONFIG7L,_CONFIG7H};
#pragma udata


/** P R I V A T E  P R O T O T Y P E S ***************************************/
static void InitializeSystem(void);
void USBTasks(void);
void ProcessIO(unsigned char*);
unsigned char data[8]={0,0,0,0,0,0,0,0};

/** V E C T O R  R E M A P P I N G *******************************************/

extern void _startup (void);        // See c018i.c in your C18 compiler dir
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800
void _reset (void)
{
    _asm goto _startup _endasm
}
#pragma code

/** D E C L A R A T I O N S **************************************************/
#pragma code

//Ms 
#define IR_IN PORTBbits.RB0
//ԊOM֐ 
short int recieve(void){
	unsigned char cnt,cnt1,cnt2;
	for (cnt1=0;cnt1<4;cnt1++){
		for (cnt2=0;cnt2<8;cnt2++){
			while (!IR_IN){};		//҂
			for (cnt=0;IR_IN;){	//pXv70.3125us
				Delay100TCYx(8);
				Delay10TCYx(4);
				cnt++;
				if (!cnt)				//^CAEg
					return(1);			//G[Ԃ
			}
			data[cnt1]<<=1;				//1bitVtg
			if (cnt<=5)					//ZƂ
				return (1);				//G[Ԃ
			if ((5<cnt)&&(cnt<=14))		//562.5usÔƂ
				data[cnt1]&=0xFE;
//				bit_clear(data[cnt1],0);	//Bit00ɂ
			if ((14<cnt)&&(cnt<=42))		//1687.5usÔƂ
				data[cnt1]|=0x01;
//				bit_set(data[cnt1],0);		//Bit01ɂ
			if (42<cnt)					//Ƃ
				return (1);				//G[Ԃ
		}
	}
	return (0);							//I
}


/******************************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *****************************************************************************/
#define TMAI 256*64
void main(void)
{
	unsigned int tma=0;
	unsigned char cnt;
    InitializeSystem();
    while(1)
    {
		USBTasks();        // USB Tasks
		while (!IR_IN){USBTasks();};	//҂
		while (IR_IN){
			USBTasks();
			Delay10TCYx(3);
			tma--;
			if (tma==0){
				data[0]=0xFF;
				data[1]=0xFF;
				data[2]=0xFF;
				data[3]=0xFF;
				ProcessIO(data);
			}
		}	//オ҂
		for (cnt=0;!IR_IN;){			//ONԌv
			Delay100TCYx(8);
			Delay10TCYx(4);
			cnt++;
			if (!cnt) continue;		//^CAEg
		}
		if ((cnt<=90)||(181<cnt))		//9msOłȂ~
			continue;
		for (cnt=0;IR_IN;){			//OFFԌv
			Delay100TCYx(8);
			Delay10TCYx(4);
			cnt++;
			if (!cnt) continue;		//^CAEg
		}
		if	(cnt<45){				//s[gR[h
			data[0]=0;
			data[1]=0;
			data[2]=0;
			data[3]=0;
			ProcessIO(data);
			tma=TMAI;
			continue;
		}
		if (recieve()){				//{M
			continue;
		}
		ProcessIO(data);
		tma=TMAI;
    }//end while
}//end main

/******************************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization routine.
 *                  All required USB initialization routines are called from
 *                  here.
 *
 *                  User application initialization routine should also be
 *                  called from here.                  
 *
 * Note:            None
 *****************************************************************************/
static void InitializeSystem(void)
{
	TRISA=0x00;
	TRISB=0x01;
	TRISC=0x00;
	ADCON1 |= 0x0F;                 // Default all pins to digital
	OSCCON=0;
    
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See io_cfg.h
    #endif
    
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;
    #endif
    
    mInitializeUSBDriver();         // See usbdrv.h
    
    //UserInit();                     // See user.c & .h

}//end InitializeSystem

/******************************************************************************
 * Function:        void USBTasks(void)
 *
 * PreCondition:    InitializeSystem has been called.
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Service loop for USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void USBTasks(void)
{
    /*
     * Servicing Hardware
     */
    USBCheckBusStatus();                    // Must use polling method
    if(UCFGbits.UTEYE!=1)
        USBDriverService();                 // Interrupt or polling method

}// end USBTasks

/** EOF main.c ***************************************************************/
